@@ -4,6 +4,7 @@
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
 #undef HAVE_DEFOPEN
+#undef HAVE_FBCONSOLE
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
@@ -27,8 +28,13 @@
 #undef EXPANDED_GDMCONFIGDIR
 #undef EXPANDED_LOCALEDIR
 #undef EXPANDED_AUTHDIR
+#undef EXPANDED_LOGDIR
 #undef EXPANDED_SYSCONFDIR
 #undef EXPANDED_SESSDIR
+#undef X_SERVER
+#undef X_SERVER_PATH
+#undef X_CONF_PATH
 #undef GETTEXT_PACKAGE
 #undef HAVE_LOGINCAP
 #undef HAVE_SYS_SOCKIO_H
+#undef HAVE_SCHED_YIELD
