@@ -29,6 +29,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <sys/resource.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -55,6 +56,7 @@
 #include "choose.h"
 #include "getvt.h"
 #include "gdm-net.h"
+#include "cookie.h"
 
 /* Local functions */
 static void gdm_config_parse (void);
@@ -81,6 +83,8 @@ pid_t extra_process = 0;	/* An extra process.  Used for quickie
 				   processes, so that they also get whacked */
 int extra_status = 0;		/* Last status from the last extra process */
 pid_t gdm_main_pid = 0;		/* PID of the main daemon */
+
+gboolean print_version = FALSE; /* print version number and quit */
 gboolean preserve_ld_vars = FALSE; /* Preserve the ld environment variables */
 gboolean no_daemon = FALSE;	/* Do not daemonize */
 gboolean no_console = FALSE;	/* There are no local servers, this means,
@@ -112,9 +116,9 @@ gboolean gdm_in_final_cleanup = FALSE;
 /* Configuration options */
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
+gchar *GdmGtkRC = NULL;
 gchar *GdmSessDir = NULL;
 gchar *GdmXsession = NULL;
-gchar *GdmLocaleFile = NULL;
 gchar *GdmAutomaticLogin = NULL;
 gboolean GdmAutomaticLoginEnable = FALSE;
 gboolean GdmAlwaysRestartServer = FALSE;
@@ -154,7 +158,6 @@ gchar *GdmDefaultPath = NULL;
 gchar *GdmRootPath = NULL;
 gboolean  GdmKillInitClients = FALSE;
 gint  GdmUserMaxFile = 0;
-gint  GdmSessionMaxFile = 0;
 gboolean  GdmXdmcp = FALSE;
 gint  GdmDispPerHost = 0;
 gint  GdmMaxPending = 0;
@@ -226,7 +229,7 @@ check_servauthdir (struct stat *statbuf)
 
     /* Enter paranoia mode */
     IGNORE_EINTR (r = stat (GdmServAuthDir, statbuf));
-    if (r < 0) {
+    if G_UNLIKELY (r < 0) {
 	    char *s = g_strdup_printf
 		    (_("Server Authorization directory "
 		       "(daemon/ServAuthDir) is set to %s "
@@ -240,7 +243,7 @@ check_servauthdir (struct stat *statbuf)
 	    gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 
-    if (! S_ISDIR (statbuf->st_mode)) {
+    if G_UNLIKELY (! S_ISDIR (statbuf->st_mode)) {
 	    char *s = g_strdup_printf
 		    (_("Server Authorization directory "
 		       "(daemon/ServAuthDir) is set to %s "
@@ -291,6 +294,11 @@ gdm_config_parse (void)
     displays = NULL;
     high_display_num = 0;
 
+#if defined(_SCO_DS) || defined(__UNIXWARE__) || defined(__USLC__)
+    gdm_error ("SCO OS detected!!!  I will sleep for 5 seconds in protest.  This message can be disabled by SCO stopping to sue people.");
+    sleep (5);
+#endif
+
     IGNORE_EINTR (r = stat (GDM_CONFIG_FILE, &statbuf));
     if (r < 0) {
 	    gdm_error (_("%s: No configuration file: %s. Using defaults."),
@@ -342,16 +350,17 @@ gdm_config_parse (void)
 			"gdm_config_parse",
 			EXPANDED_SYSCONFDIR);
 	    g_free (GdmXsession);
-	    GdmXsession = g_strconcat (EXPANDED_SYSCONFDIR, "/gdm/Xsession",
-				       NULL);
+	    GdmXsession = g_build_filename (EXPANDED_SYSCONFDIR,
+					    "gdm", "Xsession", NULL);
     }
     GdmSuspend = ve_config_get_string (cfg, GDM_KEY_SUSPEND);
-    GdmLocaleFile = ve_config_get_string (cfg, GDM_KEY_LOCFILE);
     GdmUser = ve_config_get_string (cfg, GDM_KEY_USER);
     GdmUserAuthDir = ve_config_get_string (cfg, GDM_KEY_UAUTHDIR);
     GdmUserAuthFile = ve_config_get_string (cfg, GDM_KEY_UAUTHFILE);
     GdmUserAuthFB = ve_config_get_string (cfg, GDM_KEY_UAUTHFB);
 
+    GdmGtkRC = ve_config_get_string (cfg, GDM_KEY_GTKRC);
+
     GdmTimedLoginEnable = ve_config_get_bool (cfg, GDM_KEY_TIMED_LOGIN_ENABLE);
     GdmTimedLogin = ve_config_get_string (cfg, GDM_KEY_TIMED_LOGIN);
     GdmTimedLoginDelay = ve_config_get_int (cfg, GDM_KEY_TIMED_LOGIN_DELAY);
@@ -361,7 +370,6 @@ gdm_config_parse (void)
     GdmAllowRemoteAutoLogin = ve_config_get_bool (cfg, GDM_KEY_ALLOWREMOTEAUTOLOGIN);
     GdmRelaxPerms = ve_config_get_int (cfg, GDM_KEY_RELAXPERM);
     GdmUserMaxFile = ve_config_get_int (cfg, GDM_KEY_MAXFILE);
-    GdmSessionMaxFile = ve_config_get_int (cfg, GDM_KEY_SESSIONMAXFILE);
 
     GdmXdmcp = ve_config_get_bool (cfg, GDM_KEY_XDMCP);
     GdmDispPerHost = ve_config_get_int (cfg, GDM_KEY_DISPERHOST);
@@ -547,7 +555,7 @@ gdm_config_parse (void)
     }
     ve_config_free_list_of_strings (list);
 
-    if (displays == NULL && ! GdmXdmcp) {
+    if G_UNLIKELY (displays == NULL && ! GdmXdmcp) {
 	    char *server = NULL;
 
 	    /* if we requested no local servers (there is no console),
@@ -604,14 +612,14 @@ gdm_config_parse (void)
     /* Lookup user and groupid for the gdm user */
     pwent = getpwnam (GdmUser);
 
-    if (pwent == NULL) {
+    if G_UNLIKELY (pwent == NULL) {
 	    gdm_error (_("%s: Can't find the gdm user (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmUser);
 	    GdmUser = g_strdup ("nobody");
 	    pwent = getpwnam (GdmUser);
     }
 
-    if (pwent == NULL) {
+    if G_UNLIKELY (pwent == NULL) {
 	    char *s = g_strdup_printf
 		    (_("The gdm user does not exist. "
 		       "Please correct gdm configuration %s "
@@ -625,7 +633,7 @@ gdm_config_parse (void)
 	    GdmUserId = pwent->pw_uid;
     }
 
-    if (GdmUserId == 0) {
+    if G_UNLIKELY (GdmUserId == 0) {
 	    char *s = g_strdup_printf
 		    (_("The gdm user is set to be root, but "
 		       "this is not allowed since it can "
@@ -640,14 +648,14 @@ gdm_config_parse (void)
 
     grent = getgrnam (GdmGroup);
 
-    if (grent == NULL) {
+    if G_UNLIKELY (grent == NULL) {
 	    gdm_error (_("%s: Can't find the gdm group (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmGroup);
 	    GdmGroup = g_strdup ("nobody");
 	    grent = getgrnam (GdmGroup);
     }
 
-    if (grent == NULL) {
+    if G_UNLIKELY (grent == NULL) {
 	    char *s = g_strdup_printf
 		    (_("The gdm group does not exist. "
 		       "Please correct gdm configuration %s "
@@ -661,7 +669,7 @@ gdm_config_parse (void)
 	    GdmGroupId = grent->gr_gid;   
     }
 
-    if (GdmGroupId == 0) {
+    if G_UNLIKELY (GdmGroupId == 0) {
 	    char *s = g_strdup_printf
 		    (_("The gdm group is set to be root, but "
 		       "this is not allowed since it can "
@@ -684,15 +692,15 @@ gdm_config_parse (void)
 
     /* Check that the greeter can be executed */
     bin = ve_first_word (GdmGreeter);
-    if (ve_string_empty (bin) ||
-	access (bin, X_OK) != 0) {
+    if G_UNLIKELY (ve_string_empty (bin) ||
+		   access (bin, X_OK) != 0) {
 	    gdm_error (_("%s: Greeter not found or can't be executed by the gdm user"), "gdm_config_parse");
     }
     g_free (bin);
 
     bin = ve_first_word (GdmRemoteGreeter);
-    if (ve_string_empty (bin) ||
-	access (bin, X_OK) != 0) {
+    if G_UNLIKELY (ve_string_empty (bin) ||
+		   access (bin, X_OK) != 0) {
 	    gdm_error (_("%s: Remote greeter not found or can't be executed by the gdm user"), "gdm_config_parse");
     }
     g_free (bin);
@@ -701,16 +709,16 @@ gdm_config_parse (void)
     /* Check that chooser can be executed */
     bin = ve_first_word (GdmChooser);
 
-    if (GdmIndirect &&
-	(ve_string_empty (bin) ||
-	 access (bin, X_OK) != 0)) {
+    if G_UNLIKELY (GdmIndirect &&
+		   (ve_string_empty (bin) ||
+		    access (bin, X_OK) != 0)) {
 	    gdm_error (_("%s: Chooser not found or it can't be executed by the gdm user"), "gdm_config_parse");
     }
     
     g_free (bin);
 
     /* Check the serv auth and log dirs */
-    if (ve_string_empty (GdmServAuthDir)) {
+    if G_UNLIKELY (ve_string_empty (GdmServAuthDir)) {
 	    if ( ! no_console)
 		    gdm_text_message_dialog
 			    (_("No daemon/ServAuthDir specified in the configuration file"));
@@ -739,7 +747,7 @@ gdm_config_parse (void)
     /* again paranoid */
     check_servauthdir (&statbuf);
 
-    if (statbuf.st_uid != 0 || statbuf.st_gid != GdmGroupId)  {
+    if G_UNLIKELY (statbuf.st_uid != 0 || statbuf.st_gid != GdmGroupId)  {
 	    char *s = g_strdup_printf
 		    (_("Server Authorization directory "
 		       "(daemon/ServAuthDir) is set to %s "
@@ -756,7 +764,7 @@ gdm_config_parse (void)
 		      GdmServAuthDir, gdm_root_user (), GdmGroup);
     }
 
-    if (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRWXG|S_ISVTX))  {
+    if G_UNLIKELY (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRWXG|S_ISVTX))  {
 	    char *s = g_strdup_printf
 		    (_("Server Authorization directory "
 		       "(daemon/ServAuthDir) is set to %s "
@@ -819,19 +827,36 @@ gdm_daemonify (void)
     pid = fork ();
     if (pid > 0) {
 
+        errno = 0;
 	if ((pf = gdm_safe_fopen_w (GdmPidFile)) != NULL) {
+	    errno = 0;
 	    fprintf (pf, "%d\n", (int)pid);
 	    fclose (pf);
+	    if (errno != 0) {
+		    /* FIXME: how to handle this? */
+		    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
+				  GdmPidFile, strerror (errno));
+		    gdm_error ("Cannot write PID file %s, possibly out of diskspace.  Error: %s",
+			       GdmPidFile, strerror (errno));
+
+	    }
+	} else if (errno != 0) {
+	    /* FIXME: how to handle this? */
+	    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
+			  GdmPidFile, strerror (errno));
+	    gdm_error ("Cannot write PID file %s, possibly out of diskspace.  Error: %s",
+		       GdmPidFile, strerror (errno));
+
 	}
 
         exit (EXIT_SUCCESS);
     }
     gdm_main_pid = getpid ();
 
-    if (pid < 0) 
+    if G_UNLIKELY (pid < 0) 
 	gdm_fail (_("%s: fork() failed!"), "gdm_daemonify");
 
-    if (setsid() < 0)
+    if G_UNLIKELY (setsid() < 0)
 	gdm_fail (_("%s: setsid() failed: %s!"), "gdm_daemonify",
 		  strerror(errno));
 
@@ -962,7 +987,7 @@ gdm_final_cleanup (void)
 	if (fifoconn != NULL) {
 		char *path;
 		gdm_connection_close (fifoconn);
-		path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
+		path = g_build_filename (GdmServAuthDir, ".gdmfifo", NULL);
 		IGNORE_EINTR (unlink (path));
 		g_free (path);
 		fifoconn = NULL;
@@ -986,8 +1011,9 @@ gdm_final_cleanup (void)
 
 	closelog();
 
-	if (GdmPidFile != NULL)
+	if (GdmPidFile != NULL) {
 		IGNORE_EINTR (unlink (GdmPidFile));
+	}
 }
 
 static gboolean
@@ -1029,7 +1055,7 @@ deal_with_x_crashes (GdmDisplay *d)
 
 	    if (pid == 0) {
 		    char *argv[2];
-		    char *xlog = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
+		    char *xlog = gdm_make_filename (GdmLogDir, d->name, ".log");
 
 		    gdm_unset_signals ();
 
@@ -1211,18 +1237,26 @@ gdm_cleanup_children (void)
     if (pid <= 0)
 	    return FALSE;
 
-    if (WIFEXITED (exitstatus)) {
+    if G_LIKELY (WIFEXITED (exitstatus)) {
 	    status = WEXITSTATUS (exitstatus);
 	    crashed = FALSE;
 	    gdm_debug ("gdm_cleanup_children: child %d returned %d", pid, status);
     } else {
 	    status = EXIT_SUCCESS;
 	    crashed = TRUE;
-	    if (WIFSIGNALED (exitstatus))
-		    gdm_debug ("gdm_cleanup_children: child %d crashed of signal %d", pid,
-			       (int)WTERMSIG (exitstatus));
-	    else
-		    gdm_debug ("gdm_cleanup_children: child %d crashed", pid);
+	    if (WIFSIGNALED (exitstatus)) {
+		    if (WTERMSIG (exitstatus) == SIGTERM ||
+			WTERMSIG (exitstatus) == SIGINT) {
+			    /* we send these signals, sometimes children don't handle them */
+			    gdm_debug ("gdm_cleanup_children: child %d died of signal %d (TERM/INT)", pid,
+				       (int)WTERMSIG (exitstatus));
+		    } else {
+			    gdm_error ("gdm_cleanup_children: child %d crashed of signal %d", pid,
+				       (int)WTERMSIG (exitstatus));
+		    }
+	    } else {
+		    gdm_error ("gdm_cleanup_children: child %d crashed", pid);
+	    }
     }
 	
     if (pid == extra_process) {
@@ -1238,8 +1272,8 @@ gdm_cleanup_children (void)
     if (d == NULL)
 	    return TRUE;
 
-    if (crashed) {
-	    gdm_debug ("gdm_cleanup_children: Slave crashed, killing its "
+    if G_UNLIKELY (crashed) {
+	    gdm_error ("gdm_cleanup_children: Slave crashed, killing its "
 		       "children");
 
 	    if (d->sesspid > 1)
@@ -1533,9 +1567,21 @@ gdm_safe_restart (void)
 	gdm_restart_now ();
 }
 
+static void
+main_daemon_abrt (int sig)
+{
+	/* FIXME: note that this could mean out of memory */
+	gdm_error ("main daemon: Got SIGABRT, something went very wrong. Going down!");
+	gdm_final_cleanup ();
+	exit (EXIT_FAILURE);
+}
+
 static gboolean
 mainloop_sig_callback (int sig, gpointer data)
 {
+  /* signals are at somewhat random times aren't they? */
+  gdm_random_tick ();
+
   gdm_debug ("mainloop_sig_callback: Got signal %d", (int)sig);
   switch (sig)
     {
@@ -1551,6 +1597,20 @@ mainloop_sig_callback (int sig, gpointer data)
       exit (EXIT_SUCCESS);
       break;
 
+#ifdef SIGXFSZ
+    case SIGXFSZ:
+      gdm_error ("main daemon: Hit file size rlimit, restarting!");
+      gdm_restart_now ();
+      break;
+#endif
+
+#ifdef SIGXCPU
+    case SIGXCPU:
+      gdm_error ("main daemon: Hit CPU rlimit, restarting!");
+      gdm_restart_now ();
+      break;
+#endif
+
     case SIGHUP:
       gdm_restart_now ();
       break;
@@ -1596,11 +1656,11 @@ create_connections (void)
 	int p[2];
 	gchar *path;
 
-	path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
+	path = g_build_filename (GdmServAuthDir, ".gdmfifo", NULL);
 	fifoconn = gdm_connection_open_fifo (path, 0660);
 	g_free (path);
 
-	if (fifoconn != NULL) {
+	if G_LIKELY (fifoconn != NULL) {
 		gdm_connection_set_handler (fifoconn,
 					    gdm_handle_message,
 					    NULL /* data */,
@@ -1610,7 +1670,7 @@ create_connections (void)
 						 close_notify);
 	}
 
-	if (pipe (p) < 0) {
+	if G_UNLIKELY (pipe (p) < 0) {
 		slave_fifo_pipe_fd = -1;
 		pipeconn = NULL;
 	} else {
@@ -1618,7 +1678,7 @@ create_connections (void)
 		pipeconn = gdm_connection_open_fd (p[0]);
 	}
 
-	if (pipeconn != NULL) {
+	if G_LIKELY (pipeconn != NULL) {
 		gdm_connection_set_handler (pipeconn,
 					    gdm_handle_message,
 					    NULL /* data */,
@@ -1635,7 +1695,7 @@ create_connections (void)
 
 	unixconn = gdm_connection_open_unix (GDM_SUP_SOCKET, 0666);
 
-	if (unixconn != NULL) {
+	if G_LIKELY (unixconn != NULL) {
 		gdm_connection_set_handler (unixconn,
 					    gdm_handle_user_message,
 					    NULL /* data */,
@@ -1653,6 +1713,8 @@ struct poptOption options [] = {
 	  &no_console, 0, N_("No console (local) servers to be run"), NULL },
 	{ "preserve-ld-vars", '\0', POPT_ARG_NONE,
 	  &preserve_ld_vars, 0, N_("Preserve LD_* variables"), NULL },
+	{ "version", '\0', POPT_ARG_NONE,
+	  &print_version, 0, NULL, NULL },
         POPT_AUTOHELP
 	{ NULL, 0, 0, NULL, 0}
 };
@@ -1711,11 +1773,13 @@ gdm_get_our_runlevel (void)
 	if (access ("/sbin/runlevel", X_OK) == 0) {
 		char ign;
 		int rnl;
-		FILE *fp = fopen ("/sbin/runlevel", "r");
-		if (fscanf (fp, "%c %d", &ign, &rnl) == 2) {
-			gdm_normal_runlevel = rnl;
+		FILE *fp = popen ("/sbin/runlevel", "r");
+		if (fp != NULL) {
+			if (fscanf (fp, "%c %d", &ign, &rnl) == 2) {
+				gdm_normal_runlevel = rnl;
+			}
+			pclose (fp);
 		}
-		fclose (fp);
 	}
 #endif /* __linux__ */
 }
@@ -1734,20 +1798,23 @@ int
 main (int argc, char *argv[])
 {
     sigset_t mask;
-    struct sigaction sig, child;
+    struct sigaction sig, child, abrt;
     FILE *pf;
     poptContext ctx;
     int nextopt;
     const char *charset;
 
+    gdm_random_tick ();
+
     gdm_main_pid = getpid ();
 
     /* We here ensure descriptors 0, 1 and 2 */
     ensure_desc_012 ();
 
+    /* store all initial stuff, args, env, rlimits, runlevel */
     store_argv (argc, argv);
     gdm_saveenv ();
-
+    gdm_get_initial_limits ();
     gdm_get_our_runlevel ();
 
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -1763,18 +1830,24 @@ main (int argc, char *argv[])
     while ((nextopt = poptGetNextOpt (ctx)) > 0 || nextopt == POPT_ERROR_BADOPT)
 	/* do nothing */ ;
 
-    if (nextopt != -1) {
+    if G_UNLIKELY (nextopt != -1) {
 	    fprintf (stderr,
 		     _("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
 		     poptBadOption (ctx, 0),
 		     poptStrerror (nextopt),
 		     argv[0]);
-	    fflush (stdout);
+	    fflush (stderr);
 	    exit (1);
     }
 
+    if (print_version) {
+	    printf ("GDM %s\n", VERSION);
+	    fflush (stdout);
+	    exit (0);
+    }
+
     /* XDM compliant error message */
-    if (getuid () != 0) {
+    if G_UNLIKELY (getuid () != 0) {
 	    /* make sure the pid file doesn't get wiped */
 	    GdmPidFile = NULL;
 	    gdm_fail (_("Only root wants to run gdm\n"));
@@ -1792,11 +1865,11 @@ main (int argc, char *argv[])
     sig.sa_flags = SA_RESTART;
     sigemptyset (&sig.sa_mask);
 
-    if (sigaction (SIGTERM, &sig, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGTERM, &sig, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
 		  "main", "TERM", strerror (errno));
 
-    if (sigaction (SIGINT, &sig, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGINT, &sig, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
 		  "main", "INT", strerror (errno));
 
@@ -1832,9 +1905,26 @@ main (int argc, char *argv[])
     if (no_daemon || getppid() == 1) {
 
 	/* Write pid to pidfile */
+	errno = 0;
 	if ((pf = gdm_safe_fopen_w (GdmPidFile)) != NULL) {
+	    errno = 0;
 	    fprintf (pf, "%d\n", (int)getpid());
 	    fclose (pf);
+	    if (errno != 0) {
+		    /* FIXME: how to handle this? */
+		    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
+				  GdmPidFile, strerror (errno));
+		    gdm_error ("Cannot write PID file %s, possibly out of diskspace.  Error: %s",
+			       GdmPidFile, strerror (errno));
+
+	    }
+	} else if (errno != 0) {
+	    /* FIXME: how to handle this? */
+		gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
+			      GdmPidFile, strerror (errno));
+	    gdm_error ("Cannot write PID file %s, possibly out of diskspace.  Error: %s",
+		       GdmPidFile, strerror (errno));
+
 	}
 
 	IGNORE_EINTR (chdir (GdmServAuthDir));
@@ -1849,33 +1939,60 @@ main (int argc, char *argv[])
     ve_signal_add (SIGINT, mainloop_sig_callback, NULL);
     ve_signal_add (SIGHUP, mainloop_sig_callback, NULL);
     ve_signal_add (SIGUSR1, mainloop_sig_callback, NULL);
-    
+
     sig.sa_handler = ve_signal_notify;
     sig.sa_flags = SA_RESTART;
     sigemptyset (&sig.sa_mask);
 
-    if (sigaction (SIGTERM, &sig, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGTERM, &sig, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
 		  "main", "TERM", strerror (errno));
 
-    if (sigaction (SIGINT, &sig, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGINT, &sig, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
 		  "main", "INT", strerror (errno));
 
-    if (sigaction (SIGHUP, &sig, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGHUP, &sig, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
 		  "main", "HUP", strerror (errno));
 
-    if (sigaction (SIGUSR1, &sig, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGUSR1, &sig, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
 		  "main", "USR1", strerror (errno));
 
+    /* some process limit signals we catch and restart on,
+       note that we don't catch these in the slave, but then
+       we catch those in the main daemon as slave crashing
+       (terminated by signal), and we clean up appropriately */
+#ifdef SIGXCPU
+    ve_signal_add (SIGXCPU, mainloop_sig_callback, NULL);
+    if G_UNLIKELY (sigaction (SIGXCPU, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "XCPU", strerror (errno));
+#endif
+#ifdef SIGXFSZ
+    ve_signal_add (SIGXFSZ, mainloop_sig_callback, NULL);
+    if G_UNLIKELY (sigaction (SIGXFSZ, &sig, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "XFSZ", strerror (errno));
+#endif
+
+    /* cannot use mainloop for SIGABRT, the handler can never
+       return */
+    abrt.sa_handler = main_daemon_abrt;
+    abrt.sa_flags = SA_RESTART;
+    sigemptyset (&abrt.sa_mask);
+
+    if G_UNLIKELY (sigaction (SIGABRT, &abrt, NULL) < 0) 
+	gdm_fail (_("%s: Error setting up %s signal handler: %s"),
+		  "main", "ABRT", strerror (errno));
+
     child.sa_handler = ve_signal_notify;
     child.sa_flags = SA_RESTART|SA_NOCLDSTOP;
     sigemptyset (&child.sa_mask);
     sigaddset (&child.sa_mask, SIGCHLD);
 
-    if (sigaction (SIGCHLD, &child, NULL) < 0) 
+    if G_UNLIKELY (sigaction (SIGCHLD, &child, NULL) < 0) 
 	gdm_fail (_("%s: Error setting up CHLD signal handler"), "gdm_main");
 
     sigemptyset (&mask);
@@ -1884,10 +2001,26 @@ main (int argc, char *argv[])
     sigaddset (&mask, SIGCHLD);
     sigaddset (&mask, SIGHUP);
     sigaddset (&mask, SIGUSR1);
+    sigaddset (&mask, SIGABRT);
+#ifdef SIGXCPU
+    sigaddset (&mask, SIGXCPU);
+#endif
+#ifdef SIGXFSZ
+    sigaddset (&mask, SIGXFSZ);
+#endif
     sigprocmask (SIG_UNBLOCK, &mask, NULL);
 
     gdm_signal_ignore (SIGUSR2);
-    gdm_signal_ignore (SIGPIPE);
+
+    /* ignore power failures, up to user processes to
+     * handle things correctly */
+#ifdef SIGPWR
+    gdm_signal_ignore (SIGPWR);
+#endif
+    /* can we ever even get this one? */
+#ifdef SIGLOST
+    gdm_signal_ignore (SIGLOST);
+#endif
 
     gdm_debug ("gdm_main: Here we go...");
 
@@ -1897,6 +2030,9 @@ main (int argc, char *argv[])
 
     create_connections ();
 
+    /* make sure things (currently /tmp/.ICE-unix) are sane */
+    gdm_ensure_sanity () ;
+
     /* Start local X servers */
     gdm_start_first_unborn_local (0 /* delay */);
 
@@ -1952,7 +2088,7 @@ static void
 write_x_servers (GdmDisplay *d)
 {
 	FILE *fp;
-	char *file = g_strconcat (GdmServAuthDir, "/", d->name, ".Xservers", NULL);
+	char *file = gdm_make_filename (GdmServAuthDir, d->name, ".Xservers");
 	int i;
 	int bogusname;
 
@@ -1960,7 +2096,7 @@ write_x_servers (GdmDisplay *d)
 		d->x_servers_order = get_new_order (d);
 
 	fp = gdm_safe_fopen_w (file);
-	if (fp == NULL) {
+	if G_UNLIKELY (fp == NULL) {
 		gdm_error ("Can't open %s for writing", file);
 		g_free (file);
 		return;
@@ -1988,6 +2124,7 @@ write_x_servers (GdmDisplay *d)
 		g_free (command);
 	}
 
+	/* FIXME: What about out of disk space errors? */
 	fclose (fp);
 }
 
@@ -2043,7 +2180,7 @@ static void
 gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 {
 	/* Evil!, all this for debugging? */
-	if (GdmDebug) {
+	if G_UNLIKELY (GdmDebug) {
 		if (strncmp (msg, GDM_SOP_COOKIE " ",
 			     strlen (GDM_SOP_COOKIE " ")) == 0) {
 			char *s = g_strndup
@@ -2583,7 +2720,7 @@ extract_dispnum (const char *addy)
 	int num;
 	char *p;
 
-	g_assert (addy != NULL);
+	gdm_assert (addy != NULL);
 
 	p = strchr (addy, ':');
 	if (p == NULL)
@@ -2689,9 +2826,9 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 
 		seteuid (xnest_uid);
 
-		g_assert (xnest_auth_file != NULL);
-		g_assert (xnest_disp != NULL);
-		g_assert (xnest_cookie != NULL);
+		gdm_assert (xnest_auth_file != NULL);
+		gdm_assert (xnest_disp != NULL);
+		gdm_assert (xnest_cookie != NULL);
 
 		IGNORE_EINTR (r = stat (xnest_auth_file, &s));
 		if (r < 0)
@@ -2741,7 +2878,7 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 	g_free (bin);
 
 	display = gdm_server_alloc (-1, server);
-	if (display == NULL) {
+	if G_UNLIKELY (display == NULL) {
 		gdm_connection_write (conn,
 				      "ERROR 2 Startup errors\n");
 		return;
@@ -2756,7 +2893,7 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 	if (type == TYPE_FLEXI_XNEST) {
 		GdmDisplay *parent;
 		char *disp, *p;
-		g_assert (xnest_disp != NULL);
+		gdm_assert (xnest_disp != NULL);
 
 		disp = g_strdup (xnest_disp);
 		/* whack the screen info */
@@ -3135,13 +3272,13 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 
 		svr = gdm_find_x_server (name);
-		if (svr == NULL) {
+		if G_UNLIKELY (svr == NULL) {
 			/* Don't print the name to syslog as it might be
 			 * long and dangerous */
 			gdm_error (_("Unknown server type requested, using "
 				     "standard server."));
 			command = GdmStandardXServer;
-		} else if ( ! svr->flexible) {
+		} else if G_UNLIKELY ( ! svr->flexible) {
 			gdm_error (_("Requested server %s not allowed to be "
 				     "used for flexible servers, using "
 				     "standard server."), name);
