@@ -278,6 +278,7 @@ struct _GdmDisplay {
     gchar *authfile_gdm; /* authfile readable by gdm user
 			    if necessary */
     GSList *auths; 
+    GSList *local_auths; 
     gchar *userauth;
     gboolean authfb;
     gchar *command;
@@ -312,6 +313,7 @@ struct _GdmDisplay {
     time_t acctime;
 
     gboolean handled;
+    gboolean tcp_disallowed;
 
     int vt;
 
@@ -364,6 +366,8 @@ struct _GdmDisplay {
     int xsession_errors_bytes;
 #define MAX_XSESSION_ERRORS_BYTES (80*2500)  /* maximum number of bytes in
 						the ~/.xsession-errors file */
+    char *xsession_errors_filename; /* if NULL then there is no .xsession-errors
+				       file */
 };
 
 typedef struct _GdmXServer GdmXServer;
