@@ -1,8 +1,11 @@
+#include "config.h"
+
 #include <gtk/gtk.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "greeter_item.h"
+#include "greeter_parser.h"
 #include "greeter_events.h"
 
 struct CallbackInfo {
@@ -17,13 +20,15 @@ state_run (GreeterItemInfo *info,
 	   GreeterItemState old_state)
 {
   if (info->state != old_state &&
+      info->have_state & (1<<(info->state)) &&
+      info->have_state != (1<<GREETER_ITEM_STATE_NORMAL) &&
       info->item != NULL)
     {
       if (info->pixbufs[info->state])
-	gnome_canvas_item_set (info->item,
+        gnome_canvas_item_set (info->item,
 			       "pixbuf", info->pixbufs[info->state],
 			       NULL);
-      if (info->have_color[info->state])
+      if (info->have_color & (1<<(info->state)))
 	gnome_canvas_item_set (info->item,
 			       "fill_color_rgba", info->colors[info->state],
 			       NULL);
@@ -87,6 +92,27 @@ propagate_reset_state (GreeterItemInfo *info,
   g_list_foreach (info->box_children, propagate_reset_state_foreach, NULL);
 }
 
+void
+greeter_item_run_action_callback (const char *id)
+{
+  struct CallbackInfo *cb_info;
+  GreeterItemInfo *info;
+
+  g_return_if_fail (id != NULL);
+
+  if G_UNLIKELY (callback_hash == NULL)
+    return;
+
+  info = greeter_lookup_id (id);
+  if G_UNLIKELY (info == NULL)
+    return;
+
+  cb_info = g_hash_table_lookup (callback_hash, info->id);
+
+  if G_LIKELY (cb_info)
+    (*cb_info->func) (info, cb_info->user_data);
+}
+
 gint
 greeter_item_event_handler (GnomeCanvasItem *item,
 			    GdkEvent        *event,
@@ -97,7 +123,7 @@ greeter_item_event_handler (GnomeCanvasItem *item,
   GreeterItemInfo *button;
 
   info = data;
-  button = greeter_item_find_my_button (info);
+  button = info->my_button;
   if (button != NULL && button != info)
     {
       /* FIXME: this is a hack, we have not really left the container,
@@ -129,14 +155,7 @@ greeter_item_event_handler (GnomeCanvasItem *item,
     info->mouse_down = FALSE;
 
     if (info->mouse_over && info->id && callback_hash)
-      {
-	struct CallbackInfo *cb_info;
-
-	cb_info = g_hash_table_lookup (callback_hash, info->id);
-
-	if (cb_info)
-	  (*cb_info->func) (info, cb_info->user_data);
-      }	    
+      greeter_item_run_action_callback (info->id);
     break;
     
   default:
@@ -178,7 +197,7 @@ greeter_item_register_action_callback (char            *id,
 {
   struct CallbackInfo *info;
 
-  if (callback_hash == NULL)
+  if G_UNLIKELY (callback_hash == NULL)
     callback_hash = g_hash_table_new_full (g_str_hash,
 					   g_str_equal,
 					   g_free,
