@@ -6,6 +6,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include <unistd.h>
+#include <sys/utsname.h>
 
 #include "greeter_item.h"
 #include "greeter_configuration.h"
@@ -21,6 +22,7 @@ greeter_item_info_new (GreeterItemInfo *parent,
   int i;
 
   info = g_new0 (GreeterItemInfo, 1);
+
   info->item_type = type;
   info->parent = parent;
 
@@ -51,15 +53,33 @@ void
 greeter_item_info_free (GreeterItemInfo *info)
 {
   int i;
+  GList *list;
 
   for (i = 0; i < GREETER_ITEM_STATE_MAX; i++)
-    if (info->pixbufs[i])
-      g_object_unref (G_OBJECT (info->pixbufs[i]));
+    {
+      if (info->pixbufs[i])
+        g_object_unref (G_OBJECT (info->pixbufs[i]));
+      if (info->files[i])
+        g_free (info->files[i]);
+      if (info->fonts[i])
+        pango_font_description_free (info->fonts[i]);
+    }
+
+  list = info->fixed_children;
+  info->fixed_children = NULL;
+  g_list_foreach (list, (GFunc) greeter_item_info_free, NULL);
+  g_list_free (list);
+
+  list = info->box_children;
+  info->box_children = NULL;
+  g_list_foreach (list, (GFunc) greeter_item_info_free, NULL);
+  g_list_free (list);
 
   g_free (info->id);
   g_free (info->orig_text);
   g_free (info->show_type);
-  g_free (info->show_subtype);
+
+  memset (info, 0, sizeof (GreeterItemInfo));
   g_free (info);
 }
 
@@ -111,6 +131,7 @@ greeter_item_expand_text (const char *text)
   int r;
   int underline = -1;
   char buf[256];
+  struct utsname name;
 
   str = g_string_sized_new (strlen (text));
 
@@ -126,6 +147,10 @@ greeter_item_expand_text (const char *text)
 	    case '%':
 	      g_string_append (str, "%");
 	      break;
+	    case 'n':
+	      uname (&name);
+	      g_string_append (str, name.nodename);
+	      break;
 	    case 'h':
 	      buf[sizeof(buf) - 1] = '\0';
 	      r = gethostname (buf, sizeof(buf) - 1);
@@ -155,7 +180,7 @@ greeter_item_expand_text (const char *text)
 	      g_free (clock);
 	      break;
 	    case 0:
-	      g_warning ("Unescape %% at end of text\n");
+	      g_warning ("Unescaped %% at end of text\n");
 	      goto bail;
 	      break;
 	    default:
@@ -173,10 +198,8 @@ greeter_item_expand_text (const char *text)
 	  if (underline >= 0)
 	    {
 	      underline--;
-	      if (underline == 0) {
+	      if (underline == 0)
 	        g_string_append (str, "</u>");
-		underline = -1;
-	      }
 	    }
 	}
       p++;
@@ -218,27 +241,32 @@ greeter_item_is_visible (GreeterItemInfo *info)
       ! (info->show_modes & GREETER_ITEM_SHOW_REMOTE))
     return FALSE;
 
-  if ( ! GdmConfigAvailable &&
+  if (( ! GdmConfigAvailable || ! GdmSystemMenu) &&
       info->show_type != NULL &&
       strcmp (info->show_type, "config") == 0)
 	  return FALSE;
 
+  if (( ! GdmChooserButton || ! GdmSystemMenu) &&
+      info->show_type != NULL &&
+      strcmp (info->show_type, "chooser") == 0)
+	  return FALSE;
+
   if ( ! GdmSystemMenu &&
       info->show_type != NULL &&
       strcmp (info->show_type, "system") == 0)
 	  return FALSE;
 
-  if (GdmHalt == NULL &&
-      info->show_subtype != NULL &&
-      strcmp (info->show_subtype, "halt") == 0)
+  if (( ! GdmSystemMenu || GdmHalt == NULL) &&
+      info->show_type != NULL &&
+      strcmp (info->show_type, "halt") == 0)
 	  return FALSE;
-  if (GdmReboot == NULL &&
-      info->show_subtype != NULL &&
-      strcmp (info->show_subtype, "reboot") == 0)
+  if (( ! GdmSystemMenu || GdmReboot == NULL) &&
+      info->show_type != NULL &&
+      strcmp (info->show_type, "reboot") == 0)
 	  return FALSE;
-  if (GdmSuspend == NULL &&
-      info->show_subtype != NULL &&
-      strcmp (info->show_subtype, "suspend") == 0)
+  if (( ! GdmSystemMenu || GdmSuspend == NULL) &&
+      info->show_type != NULL &&
+      strcmp (info->show_type, "suspend") == 0)
 	  return FALSE;
 
   if (ve_string_empty (GdmTimedLogin) &&
@@ -248,14 +276,3 @@ greeter_item_is_visible (GreeterItemInfo *info)
 
   return TRUE;
 }
-
-GreeterItemInfo *
-greeter_item_find_my_button (GreeterItemInfo *info)
-{
-  if (info == NULL)
-    return NULL;
-  else if (info->button)
-    return info;
-  else
-    return greeter_item_find_my_button (info->parent);
-}
