@@ -30,6 +30,7 @@
 #include <glade/glade.h>
 
 #include "ve-misc.h"
+#include "ve-miscui.h"
 
 #include "glade-helper.h"
 
@@ -181,24 +182,24 @@ glade_helper_bad_interface (const char *filename, const char *widget,
 	else
 		typestring = g_strdup ("");
 	
-	dlg = gtk_message_dialog_new (NULL /* parent */,
-				      0 /* flags */,
-				      GTK_MESSAGE_ERROR,
-				      GTK_BUTTONS_CLOSE,
-				      _("An error occured while loading the user "
-					"interface\nelement %s%s from file %s.\n"
-					"Possibly the glade interface description was "
-					"corrupted.\n"
-					"%s cannot continue and will exit now.\n"
-					"You should check your "
-					"installation of %s or reinstall %s."),
-				      widget,
-				      typestring,
-				      filename,
-				      PACKAGE,
-				      PACKAGE,
-				      PACKAGE);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+	dlg = ve_hig_dialog_new (NULL /* parent */,
+				 0 /* flags */,
+				 GTK_MESSAGE_ERROR,
+				 GTK_BUTTONS_OK,
+				 _("Cannot load user interface"),
+				 _("An error occured while loading the user "
+				   "interface element %s%s from file %s.  "
+				   "Possibly the glade interface description was "
+				   "corrupted.  "
+				   "%s cannot continue and will exit now.  "
+				   "You should check your "
+				   "installation of %s or reinstall %s."),
+				 widget,
+				 typestring,
+				 filename,
+				 PACKAGE,
+				 PACKAGE,
+				 PACKAGE);
 
 	g_free (typestring);
 
@@ -228,26 +229,26 @@ glade_helper_bad_columns (const char *filename, const char *widget,
 	else
 		typestring = g_strdup ("");
 	
-	dlg = gtk_message_dialog_new (NULL /* parent */,
-				      0 /* flags */,
-				      GTK_MESSAGE_ERROR,
-				      GTK_BUTTONS_CLOSE,
-				      _("An error occured while loading the user "
-					"interface\nelement %s%s from file %s.\n"
-					"CList type widget should have %d columns.\n"
-					"Possibly the glade interface description was "
-					"corrupted.\n"
-					"%s cannot continue and will exit now.\n"
-					"You should check your "
-					"installation of %s or reinstall %s."),
-				      widget,
-				      typestring,
-				      filename,
-				      columns,
-				      PACKAGE,
-				      PACKAGE,
-				      PACKAGE);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+	dlg = ve_hig_dialog_new (NULL /* parent */,
+				 0 /* flags */,
+				 GTK_MESSAGE_ERROR,
+				 GTK_BUTTONS_OK,
+				 _("Cannot load user interface"),
+				 _("An error occured while loading the user "
+				   "interface element %s%s from file %s.  "
+				   "CList type widget should have %d columns.  "
+				   "Possibly the glade interface description was "
+				   "corrupted.  "
+				   "%s cannot continue and will exit now.  "
+				   "You should check your "
+				   "installation of %s or reinstall %s."),
+				 widget,
+				 typestring,
+				 filename,
+				 columns,
+				 PACKAGE,
+				 PACKAGE,
+				 PACKAGE);
 
 	g_free (typestring);
 
@@ -265,21 +266,22 @@ glade_helper_no_interface (const char *filename)
 {
 	GtkWidget *dlg;
 	
-	dlg = gtk_message_dialog_new (NULL /* parent */,
-				      0 /* flags */,
-				      GTK_MESSAGE_ERROR,
-				      GTK_BUTTONS_CLOSE,
-				      _("An error occured while loading the user "
-					"interface\nfrom file %s.\n"
-					"Possibly the glade interface description was "
-					"not found.\n"
-					"%s cannot continue and will exit now.\n"
-					"You should check your "
-					"installation of %s or reinstall %s."),
-				      filename,
-				      PACKAGE,
-				      PACKAGE,
-				      PACKAGE);
+	dlg = ve_hig_dialog_new (NULL /* parent */,
+				 0 /* flags */,
+				 GTK_MESSAGE_ERROR,
+				 GTK_BUTTONS_OK,
+				 _("Cannot load user interface"),
+				 _("An error occured while loading the user "
+				   "interface from file %s.  "
+				   "Possibly the glade interface description was "
+				   "not found.  "
+				   "%s cannot continue and will exit now.  "
+				   "You should check your "
+				   "installation of %s or reinstall %s."),
+				 filename,
+				 PACKAGE,
+				 PACKAGE,
+				 PACKAGE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 	g_warning (_("No interface could be loaded, BAD! (file: %s)"),
 		   filename);
@@ -340,3 +342,27 @@ glade_helper_search_gnome_dirs (gboolean search_gnome_dirs)
 	ve_search_gnome_dirs = search_gnome_dirs;
 }
 
+/* Make label surrounded by tag (if tag = "tag" add <tag>text</tag>) */
+void
+glade_helper_tagify_label (GladeXML *xml,
+			   const char *name,
+			   const char *tag)
+{
+	const char *lbl;
+	char *s;
+	GtkWidget *label = glade_helper_get (xml, name, GTK_TYPE_WIDGET);
+	if (GTK_IS_BUTTON (label)) {
+		label = GTK_BIN(label)->child;
+	}
+	if ( ! GTK_IS_LABEL (label)) {
+		glade_helper_bad_interface (xml->filename,
+					    name,
+					    GTK_TYPE_LABEL);
+		exit (1);
+	}
+	lbl = gtk_label_get_text (GTK_LABEL (label));
+	s = g_strdup_printf ("<%s>%s</%s>", tag, lbl, tag);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_label (GTK_LABEL (label), s);
+	g_free (s);
+}
