@@ -76,4 +76,12 @@ pid_t ve_waitpid_no_signal (pid_t pid, int *status, int options);
 /* Testing for existance of a certain locale */
 gboolean ve_locale_exists (const char *loc);
 
+int ve_setenv (const char *name, const char *value, gboolean overwrite);
+void ve_unsetenv (const char *name);
+void ve_clearenv (void);
+
+/* just like gnome_i18n_get_language_list (in fact if you link to
+   gnome it uses that */
+const GList * ve_i18n_get_language_list (const gchar *category_name);
+
 #endif /* VE_MISC_H */
